@@ -7,10 +7,12 @@ import psycopg2.extras
 import traceback
 import sys
 import logging
-
+from urllib.parse import urlparse, urlunparse, ParseResult
 import pytz
-from fk.utils import *
-from pprint import pformat
+import fk.utils
+import unicodedata
+from furl import furl
+import pprint
 
 logger = logging.getLogger(__name__)
 
@@ -18,7 +20,7 @@ logger = logging.getLogger(__name__)
 class DatabaseConnection:
 
     # Global connection
-    connection_instance = None
+    connection_instances = {}
 
     def __init__(self, config):
         self.config = config
@@ -28,7 +30,8 @@ class DatabaseConnection:
         self.db_password = self.config.get("db-password", None)
         self.db_database = self.config.get("db-database", None)
         self.db = None
-        connstr = f"postgres://{self.db_username}:{redact(self.db_password)}@{self.db_hostname}:{self.db_port}/{self.db_database}"
+        connstr = DatabaseConnection.config_to_url(config, do_redact=True)
+        # connstr = f"postgres://{self.db_username}:{fk.utils.redact(self.db_password)}@{self.db_hostname}:{self.db_port}/{self.db_database}"
         logger.info(f"DATABASE: {connstr}")
         self.connection_error = None
         self._prepare_db()
@@ -39,11 +42,54 @@ class DatabaseConnection:
     def is_ok(self):
         return self.db_hostname and self.db_port and self.db_username and self.db_password and self.db_database
 
+    @staticmethod
+    def config_to_url(config, do_redact=False):
+        if not config:
+            return ""
+        db_hostname = config.get("db-hostname", None)
+        db_port = config.get("db-port", None)
+        db_username = config.get("db-username", None)
+        db_password = config.get("db-password", None)
+        db_database = config.get("db-database", None)
+        if do_redact:
+            db_password = fk.utils.redact(db_password)
+        f = furl()
+        f.set(scheme="postgres", username=db_username, password=db_password, host=db_hostname, port=db_port, path=db_database)
+        return f.url
+
+    @staticmethod
+    def url_to_config(db_url):
+        f = furl(db_url)
+        if f.scheme != "postgres":
+            logger.warn("db_url URL did not have postgres scheme")
+            return {}
+        database = (f.path.segments or [""])[0]
+        # fmt:off
+        config={
+          "db-hostname": f.host
+        , "db-port": f.port
+        , "db-username": f.username
+        , "db-password": f.password
+        , "db-database": database
+        }
+        # fmt:on
+        return config
+
+    @staticmethod
+    def get_config_hash(config):
+        if not config:
+            return ""
+        key = ""
+        for i in ["hostname", "port", "username", "password", "database"]:
+            key += fk.utils.hashstr(config.get(f"db-{i}", ""))
+        return fk.utils.hashstr(key)
+
     @staticmethod
     def get_connection(config):
-        if not DatabaseConnection.connection_instance:
-            DatabaseConnection.connection_instance = DatabaseConnection(config)
-        return DatabaseConnection.connection_instance
+        key = DatabaseConnection.get_config_hash(config)
+        if not key in DatabaseConnection.connection_instances:
+            DatabaseConnection.connection_instances[key] = DatabaseConnection(config)
+        return DatabaseConnection.connection_instances[key]
 
     def _unprepare_db(self):
         if self.db:
@@ -84,8 +130,17 @@ class DatabaseConnection:
             self._unprepare_db()
         return False
 
+    def reconnect(self):
+        logger.info("###   ###   ###   ###   ###   ###   ###   #")
+        logger.info("##   ###   ###   ###   ###   ###   ###   ##")
+        logger.info("")
+        logger.info(f"Reconnecting...")
+        self._unprepare_db()
+        return self._prepare_db()
+
     def _query(self, query, data={}, mode="none"):
         cursor = None
+        try_reconnect = True
         try:
             if not self.db:
                 logger.warning("###   ###   ###   ###   ###   ###   ###   #")
@@ -129,6 +184,7 @@ class DatabaseConnection:
             logger.error(f"# Failed with:")
             if isinstance(e, psycopg2.InterfaceError):
                 logger.error(f"# + Connection error({type(e).__name__}): e={e} ")
+                try_reconnect = True
             elif isinstance(e, psycopg2.Error):
                 logger.error(f"# + e={e} ({type(e).__name__})")
                 logger.error(f"# + e.pgerror:{e.pgerror}")
@@ -165,6 +221,8 @@ class DatabaseConnection:
             logger.error(f"#######################################")
             logger.error("")
             logger.error("")
+        if try_reconnect:
+            self.reconnect()
 
     def query_many(self, query, data={}):
         return self._query(query, data, "many")
