@@ -12,6 +12,7 @@ import requests
 import pkg_resources
 from uuid import UUID
 import logging
+import pkg_resources
 
 logger = logging.getLogger(__name__)
 
@@ -402,16 +403,40 @@ def requests_session():
     return _requests_session_instance
 
 
+def walk_pkg(root: str, base_dir: str = "./", path: str = "", do_debug=False):
+    if do_debug and False:
+        logger.info(
+            f"""walk_pkg(root={root}
+, base_dir={base_dir}
+, path={path})\n"""
+        )
+    for name in pkg_resources.resource_listdir(root, path):
+        sub_path = f"{path}/{name}"
+        full_path = f"{base_dir}{sub_path}"
+        if pkg_resources.resource_isdir(root, full_path):
+            if do_debug:
+                # logger.info(f"{full_path} ------------------\n")
+                pass
+            walk_pkg(root, base_dir, sub_path, do_debug)
+        else:
+            # content = pkg_resources.resource_string(root, sub_path)
+            if do_debug:
+                logger.info(f"  +  {full_path}\n")
+            # Do something with the file content
 
-def get_package_file_content(filename):
+
+def get_package_file_content(base, filename, do_debug=True):
     """Look for file relative to package root and return it's content"""
-    content=""
-    path: str = os.path.abspath(os.path.join(os.path.dirname(__file__), "../../"))
-    full_filename = f"{path}/{filename}"
-    if pkg_resources.resource_exists(__name__, filename):
-        content = pkg_resources.resource_string(__name__, filename).decode("utf-8").strip()
-    elif os.path.exists(full_filename):
-        content = read_file(full_filename)
+    content = ""
+    package_base_path: str = os.path.abspath(os.path.join(os.path.dirname(base), "../"))
+    if do_debug:
+        logger.warn(f"package_base_path={package_base_path}")
+        logger.warn(f"filename={filename}")
+        logger.warn(f"__name__={__name__}")
+        logger.warn(f"base={base}")
+    if pkg_resources.resource_exists(base, filename):
+        logger.info(f"Fetching {filename} from package")
+        content = pkg_resources.resource_string(base, filename).decode("utf-8").strip()
     else:
         logger.warning(f"No content found for {full_filename}")
     return content
